#!/nfs-home/mjones/usr/bin/ruby -w

# Set this to your home directory (so httpd knows where to get your
# ~/.rubyphoto-conf from)
ENV['HOME'] ||= '/you/need/to/edit/dyn_resize'

require 'findbin'
$:.push FindBin::RealBin

require 'config'
require 'image_cat'
require 'photo'

require 'rp_debug'
include Debug
Debug.verbosity = 0

def main
  config = Config.new
  config.parse(ENV['HOME'] + '/.rubyphoto-conf')

  src_name, path, dst_file, category = get_paths_and_cat(config)

  photo = Photo.new(src_name, path, config)
  category.create(photo)
  fatal "failed to create " + dst_file unless test(?e, dst_file)

  output_img(dst_file)
end

def get_paths_and_cat(config)
  root_url, root_path = get_roots(config)

  url = ENV['REDIRECT_URL']
  fatal "REDIRECT_URL missing" unless url
  fatal "Invalid image prefix: " + url \
    unless url.sub!(%r!^#{Regexp.escape(root_url)}/!, '')
  fatal "invalid url: " + url \
    unless url =~ %r!^(.+)/((.+)_(\d+)x(\d+)(\.jpe?g))$!i;
  path, filename, name, width, height, ext = $1, $2, $3, $4, $5, $6

  category = get_category(config, width, height)

  src_name  = name + ext
  dst_file  = File.expand_path(url,  root_path)
  full_path = File.expand_path(path, root_path)

  return src_name, full_path, dst_file, category
end

def get_category (config, width, height)
  category = config.image_categories.find { |cat|
    width == cat.width && height == cat.height
  }
  fatal "invalid geometry " + width + 'x' + height unless category
  return category
end

def get_roots(config)
  root_url = config.get_option('dyn_root_url')
  fatal "missing config option dyn_root_url" unless root_url
  root_path = config.get_option('dyn_root_path')
  fatal "missing config option dyn_root_path" unless root_path

  return root_url, root_path
end

def output_img(filename)
  img = File.open(filename, 'r')
  fatal "open $dst_file: $!\n" unless img
  print "Content-type: image/jpeg\n"
  printf "Status: %s Condition Intercepted\n", ENV['REDIRECT_STATUS']
  print "\n"
  print img.read
  img.close
end

def fatal(msg)
  puts "Content-type: text/plain\n\n" + msg
  exit 1
end

main
