require 'image_size'
require 'myxml'

class Photo

  attr_accessor :parent, :album_index
  attr_reader :directory, :filename

  def initialize(filename, directory_path, config)
    @filename    = filename
    @directory   = directory_path
    @config      = config
    @sorter      = sorter
    @title       = ""
    @fields      = Hash.new
    @field_order = Array.new

    @album_index = nil
    @parent      = nil

    @size_possible = Hash.new
    @size_possible['IMAGE'] = true
  end

  def output_html
    @config.image_categories.each do |image_category|
      next unless size_possible?(image_category)
      new_filename   = image_category.page_filename(filename)
      image_template = ImageTemplate.get(@config)
      File.open(directory + "/" + new_filename, "w") { |file|
        file.puts image_template.pageheader(self, image_category)
        file.puts image_template.image(self, image_category)
        file.puts image_template.pagefooter
      }
    end
  end

  def page_generated(page_name, value=nil)
    return @pages[page_name] unless value
    @pages[page_name] = value
  end

  def tree
    tree = @parent.tree(true)
    tree += @parent.dots + title
  end

  def file_path
    @directory + "/" + @filename
  end

  def album_url
    link = parent.index_file_url
    link = './' if link.empty?
    return link
  end

  def next_photo
    parent.photos[album_index + 1]
  end

  def prev_photo
    return nil if album_index == 0
    parent.photos[album_index - 1]
  end

  def xml_path
    xml = file_path + '.xml'
    return xml if File.exists?(xml)

    if test(?l, file_path)
      link = File.readlink(file_path)
      if link[0] == ?/
        xml = link + '.xml'
      else
        xml = File.expand_path(@directory + '/' + link + '.xml')
      end
    end

    return File.exists?(xml) ? xml : nil
  end

  def title
    if @fields['title']
      return @fields['title']
    end
    return @filename
  end

  def set_field(name, value)
    @field_order.push(name) unless @fields.has_key?(name)
    @fields[name] = value
  end

  def get_field(name)
    return @fields[name] || ''
  end

  def fields
    return @field_order
  end

  def size_possible?(image_category)
#    return @size_possible[image_category.type_name] \
#      if @size_possible[image_category.type_name]

    cat_width  = image_category.width.to_i
    cat_height = image_category.height.to_i
    img = ImageSizeCache.open(file_path)
    width, height = img.get_dimensions
    return false if width == nil || height == nil
    if width > cat_width || height > cat_height
      # @size_possible[image_category.type_name] = true
      return true
    end

    return false
  end

  def <=>(other)
    @sorter[self, other]
  end

  def sorter
    sort_by = @config.get_option('sort_photos_by')

    descending = (sort_by.gsub!(/^reverse\s*/, '')) ? -1 : 1

    case sort_by
    when 'filename'
      proc { |x, y|
        (x.filename.downcase <=> y.filename.downcase) * descending
      }
    when 'title'
      proc { |x, y|
        (x.title.downcase <=> y.title.downcase) * descending
      }
    when 'mtime'
      proc { |x, y|
        a = File.new(x.file_path)
        b = File.new(y.file_path)
        (a.mtime <=> b.mtime) * descending
      }
    when 'date'
      proc { |x, y|
        (x.get_field('date') <=> y.get_field('date')) * descending
      }
    when 'order_field'
      proc { |x, y|
        if (x.get_field('order').empty? && y.get_field('order').empty?)
          0
        elsif x.get_field('order').empty?
          1
        elsif y.get_field('order').empty?
          -1
        else
          x.get_field('order').to_f <=> y.get_field('order').to_f
        end * descending
      }
    else
      $stderr.puts "Invalid value for 'sort_photos_by' config option: '#{sort_by}'"
      exit 1
    end
  end

  private :sorter

end

class PhotoReader < MyXMLParser

  include Debug

  attr_writer :photo, :config

  def init
    @current_text = ""
  end

  def startElement(name, attrs)
    @current_text = ""
  end

  def endElement(name)
    return if name == 'photo'
    
    if @config.photofield_allowed?(name)
      @photo.set_field(name, @current_text.strip)
    else
      debug 2, "  Ignoring element #{name} in " + filename
    end
  end

  def character(text)
    @current_text += text
  end

end

